#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>

#include "netmon_std.h"

/* network monitor
 * Monitors the bytes of traffic on a network device
 * 2006-05-14 ed AT s5h DOT net
 * Use at your own risk. Author not responsible for use.
 * $Id: netmon_linux.c 11889 2013-05-29 20:50:07Z ed $
 *
 * Updates at http://www.s5h.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#define PROCNET "/proc/net/dev"

FILE *fp;

struct interface_line *parse_line( char *line, struct interface_line *lst ); 
struct interface_line *parse_line( char *line, struct interface_line *lst ) {
	struct interface_line *il;
	struct interface_line *ptr;
	char *colon;
	char *temp = malloc(20);

	colon = strstr( line, ":" );
	if( colon ) {
		*colon = ' ';
	}

	if( sscanf( line, "%s", temp ) == 0 ) {
		fprintf( stderr, "Cannot get interface name\n" );
		exit(EXIT_FAILURE);
	}
	
	il = if_name( temp, lst );
	
	if( il == NULL ) {
		il = malloc( sizeof( struct interface_line ) );
		il->next = NULL;
		
		snprintf( il->name, 20, "%s", temp );
		ptr = find_last( lst );
		
		if( ptr == NULL ) {
			lst = il;
		}
		else {
			ptr->next = il;
		}
	}

	if( sscanf( line, "%s %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu %llu", 
				il->name, 
				&il->ibytes, &il->ipackets, &il->ierr, 
				&il->idrop, &il->ififo, &il->iframe, 
				&il->icompressed, &il->imulticast, 
				&il->obytes, &il->opackets, &il->oerr, 
				&il->odrop, &il->ofifo, &il->oframe, 
				&il->ocompressed, &il->omulticast 
				) == 0 ) {
		free(il);
		return(NULL);
	}

	free(temp);
	
	return(lst);
}

struct interface_line *getinterfaces() {
	char *contents = malloc( 1000 );
	struct interface_line *cur_list = NULL;

	if( !fp ) {
		if( ( fp = fopen( PROCNET, "r" ) ) == NULL ) {
			fprintf( stderr, "cannot open %s\n", PROCNET );
			exit( EXIT_FAILURE );
		}
	}

	if( fseek( fp, 0, SEEK_SET ) ) {
		fprintf( stderr, "error seeking in %s\n", PROCNET );
		exit( EXIT_FAILURE );
	}

	while( fgets( contents, 1000, fp ) ) {
		if( strstr( contents, ":" ) == NULL ) {
			continue;
		}

		cur_list = parse_line( contents, cur_list );
	}
	free(contents);
	return( cur_list );
}


