#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include <getopt.h>
#include <string.h>
#include <errno.h>

#include "netmon_std.h"

/* network monitor - ed neville
 * Monitors the bytes of traffic on a network device
 * 2006-05-14 ed AT s5h DOT net
 * Use at your own risk. Author not responsible for use.
 * $Id: netmon_shell.c 11889 2013-05-29 20:50:07Z ed $
 *
 * Updates at http://www.s5h.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

void print_list( struct interface_line *p, struct interface_line *pp, \
		struct interface_line *min );

void interface_cycle(void);

void list_interface_names(void);

char *human( uint64_t n, char *retstring );

void print_usage(void);

int main( int argc, char *argv[] ) {
	int c, packets = 0;
	char *interface_name = NULL;
	int option_index = 0;

	static struct option long_options[] = {
	/* These options set a flag. */
		{"help",	no_argument,		0, 'h'},
		{"list",	no_argument,		0, 'l'},
		{"packets",	no_argument,		0, 'p'},
		{"interface",	required_argument,	0, 'i'},
		{0, 0, 0, 0}
	};

	while( (c = getopt_long( argc, argv,"hlpi:", 
                   long_options, &option_index ) ) != -1 ) {

		switch( c ) {
			case 'h':
				print_usage();
				return( EXIT_SUCCESS );
				break;
			case 'l':
				list_interface_names();
				return( EXIT_SUCCESS );
				break;
			case 'p':
				packets = 1;
				break;
			case 'i':
				interface_name = strdup( optarg );
				if( errno == ENOMEM ) {
					return( EXIT_FAILURE );
				}
				break;
			default:
				print_usage();
				return( EXIT_FAILURE );
				break;
		}
	}

	if( interface_name != NULL ) {
		struct interface_line *il,*ll;
		
		il = getinterfaces();
		ll = il;

		while( il != NULL ) {

			if( strncmp( il->name, interface_name, 20 ) == 0 ) {
				if( packets == 0 ) {
					printf( "%llu\n%llu\n",
							il->ibytes,
							il->obytes );
				}
				else {
					printf( "%llu\n%llu\n",
							il->ipackets,
							il->opackets );
				}
				freeinterfaces(ll);
				exit( EXIT_SUCCESS );
			}
			il = il->next;
		}
		
		freeinterfaces(ll);	
		exit( EXIT_FAILURE );
	}

	
	interface_cycle();
	return( EXIT_SUCCESS );
}

void print_list( struct interface_line *p, struct interface_line *pp, \
		struct interface_line *min ) {
	char human_str[6][80];

	while( p != NULL ) {
		if( pp == NULL ) {
			pp = p;
		}

		printf( "%6s I:%s O:%s | I:%s O:%s | I:%s O:%s\n",
			p->name,
			human( p->ibytes, human_str[0] ),
			human( p->obytes, human_str[1] ),
			human( p->ibytes-pp->ibytes, human_str[2] ),
			human( p->obytes-pp->obytes, human_str[3] ),
			human( p->ibytes-min->ibytes, human_str[4] ),
			human( p->obytes-min->obytes, human_str[5] )
		);

		p=p->next;
		pp=pp->next;
		min=min->next;
	}

	printf( "\n\n" );
}


void interface_cycle( void ) {
	struct interface_line *il, *ll, *min[60];
	int count = 0, temp;

	ll = getinterfaces();

	for( count = 0 ; count < 60 ; count++ ) {
		min[count] = getinterfaces();
	}
	count = 0;

	while(1) {
		il = getinterfaces();
		freeinterfaces(min[count]);
		min[count] = getinterfaces();

		printf( "     [:: running total ::]   |   [:: reset/sec ::]   |  [:: reset/min ::]\n" );

		temp = count + 1;
		if( temp == 60 ) {
			temp = 0;
		}
		
		print_list( il, ll, min[temp] );

		freeinterfaces(ll);
		if( ++count == 60 ) {
			count = 0;
		}
		ll = il;
		usleep( 1000000 );
		fflush( stdout );
	}
}

void list_interface_names() {
	struct interface_line *p;

	p = getinterfaces();
	while( p != NULL ) {
		printf( "%s\n", p->name );
		p = p->next;
	}
	freeinterfaces(p);
}

char *human( uint64_t n, char *retstring ) {
	if( n > 1024*1024*1024*1024LL ) {
		snprintf( retstring, 80, "%4llu TiB", n/1024/1024/1024/1024 );
		return( retstring );
	}
	if( n > 1024*1024*1024 ) {
		snprintf( retstring, 80, "%4llu GiB", n/1024/1024/1024 );
		return( retstring );
	}
	if( n > 1024*1024 ) {
		snprintf( retstring, 80, "%4llu MiB", n/1024/1024 );
		return( retstring );
	}
	if( n > 1024 ) {
		snprintf( retstring, 80, "%4llu KiB", n/1024 );
		return( retstring );
	}

	snprintf( retstring, 80, "%4llu    ", n );

	return( retstring );
}

void print_usage(void) {
	printf( "Usage:\n"
		"netmon [-i|--interface] <interface>\n"
		"netmon [-p|--packets] (required interface name)\n"
		"netmon [-l|--list]\n"
		"netmon [-h|--help]\n"
	);
	return;
}
